//
// Written by Igor Tolmachev, IT Samples
//        mailto:support@itsamples.com
//
// Copyright (c) 2009.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. If the source code in this file is used in 
// any  commercial application then a statement along the lines of 
// "Portions copyright (c) Igor Tolmachev, 2009" must be included in 
// the startup banner, "About" box or printed documentation. An email 
// letting me know that you are using it would be nice as well. That's 
// not much to ask considering the amount of work that went into this.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy, and let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into 
// this file. 

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ThumbsCleaner
{
    public partial class MainForm : Form
    {
		#region Win API functions

        public const Int32 WM_SYSCOMMAND = 0x112;
        public const Int32 MF_SEPARATOR = 0x800;
        public const Int32 MF_BYPOSITION = 0x400;
        public const Int32 MF_STRING = 0x0;
        public const Int32 IDM_ABOUT = 1001;

        [DllImport("user32.dll")]
        private static extern IntPtr GetSystemMenu(IntPtr hWnd, bool bRevert);

        [DllImport("user32.dll")]
        private static extern bool InsertMenu(IntPtr hMenu, Int32 wPosition, Int32 wFlags, Int32 wIDNewItem, String lpNewItem);

        #endregion

        private bool m_bCancel = false;
        
        public MainForm()
        {
            InitializeComponent();
        }
        
        private void OnLoad(object sender, EventArgs e)
        {
            String strPath = System.Environment.GetFolderPath(System.Environment.SpecialFolder.System);
            strPath = Path.GetPathRoot(strPath);

            this.comboPath.Items.Add(strPath);
            this.comboPath.SelectedIndex = 0;

            IntPtr sysMenuHandle = GetSystemMenu(this.Handle, false);
            InsertMenu(sysMenuHandle, 7, MF_BYPOSITION | MF_SEPARATOR, 0, String.Empty);
            InsertMenu(sysMenuHandle, 8, MF_BYPOSITION, IDM_ABOUT, "About Thumbs Cleaner");
        }
        
        protected override void WndProc(ref Message m)
        {
            if (m.Msg == WM_SYSCOMMAND)
            {
                switch (m.WParam.ToInt32())
                {
                    case IDM_ABOUT:
                        AboutForm af = new AboutForm();
                        af.ShowDialog();
                        return;
                    default:
                        break;
                }
            }
            base.WndProc(ref m);
        }

        private void OnPathDropDown(object sender, EventArgs e)
        {
            foreach (String strItem in this.comboPath.Items)
                if (strItem == this.comboPath.Text)
                    return;

            this.comboPath.Items.Add(comboPath.Text);
        }

        private void OnButtonBrowse(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.ShowNewFolderButton = false;
            fbd.Description = "Browser for Folder where Thumbnail Database files" +
                "(Thumbs.db, ehthumbs.db, thumbcache_*.db) can be located:";

            if (DialogResult.OK == fbd.ShowDialog())
            {
                this.comboPath.Text = fbd.SelectedPath;
                this.comboPath.Items.Add(fbd.SelectedPath);
            }
        }

        private void OnButtonStart(object sender, EventArgs e)
        {
            this.buttonStart.Enabled = false;
            this.buttonStop.Enabled = true;
            m_bCancel = false;
            try
            {
                using (ThumbsCleaner.FileSystemEnumerator fse = new ThumbsCleaner.FileSystemEnumerator(comboPath.Text, "*.db", checkInclude.Checked))
                {
                    //IEnumerator<FileInfo> ien = fse.Matches().GetEnumerator();
                    //ien.Dispose();

                    this.dataGridViewFiles.Rows.Clear();

                    int nCounter = 0;
                    String strName = "";
                    foreach (FileInfo fi in fse.Matches())
                    {
                        strName = fi.Name.ToLower();
                        if (strName.IndexOf("thumbs.db") != -1 ||
                            strName.IndexOf("thumbcache_") != -1 ||
                            strName.IndexOf("ehthumbs") != -1)
                        {
                            this.dataGridViewFiles.Rows.Add();

                            DataGridViewRow dr = this.dataGridViewFiles.Rows[nCounter];
                            dr.Cells[0].Value = (nCounter + 1).ToString();
                            dr.Cells[1].Value = 1;
                            dr.Cells[2].Value = String.Format("{0,-14:#,##0}", fi.Length);
                            dr.Cells[3].Value = String.Format("{0,12:d}", fi.LastWriteTime);
                            dr.Cells[4].Value = fi.FullName;
                            nCounter++;
                        }

                        Application.DoEvents();

                        if (m_bCancel)
                            break;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                  string.Format("Exception {0}: {1}", ex.GetType().Name, ex.Message),
                  "Thumbs Cleaner",
                  MessageBoxButtons.OK,
                  MessageBoxIcon.Error
                  );
            }

            this.buttonStart.Enabled = true;
            this.buttonStop.Enabled = false;

            if (this.dataGridViewFiles.Rows.Count > 0)
            {
                this.buttonClean.Enabled = true;
                this.linkCheck.Visible = true;
                this.linkUncheck.Visible = true;
            }
            else
            {
                this.buttonClean.Enabled = false;
                this.linkCheck.Visible = false;
                this.linkUncheck.Visible = false;
            }
        }

        private void OnButtonStop(object sender, EventArgs e)
        {
            m_bCancel = true;
            this.buttonStop.Enabled = false;
        }

        private void OnButtonClean(object sender, EventArgs e)
        {
            int nCount = this.dataGridViewFiles.Rows.Count;

            String strFilePath = "";
            for (int i = 0; i < nCount; i++)
            {
                DataGridViewRow dr = this.dataGridViewFiles.Rows[i];
                if (dr != null)
                {
                    if (dr.Cells[1].Value.ToString() == "1")
                    {
                        strFilePath = dr.Cells[4].Value.ToString();

                        if (File.Exists(strFilePath))
                        {
                            try
                            {
                                File.Delete(strFilePath);
                            }
                            catch (Exception e0)
                            {
                                String strError = e0.Message;
                            }
                        }
                    }
                }
            }
            
            this.dataGridViewFiles.Rows.Clear();
            this.buttonClean.Enabled = false;
            this.linkCheck.Visible = false;
            this.linkUncheck.Visible = false;
        }

        private void OnClose(object sender, EventArgs e)
        {
            this.Close();
        }

        private void OnUncheck(object sender, EventArgs e)
        {
            int nCount = this.dataGridViewFiles.Rows.Count;

            for (int i = 0; i < nCount; i++)
            {
                DataGridViewRow dr = this.dataGridViewFiles.Rows[i];
                if (dr != null)
                    dr.Cells[1].Value = 0;
            }
        }

        private void OnCheck(object sender, EventArgs e)
        {
            int nCount = this.dataGridViewFiles.Rows.Count;

            for (int i = 0; i < nCount; i++)
            {
                DataGridViewRow dr = this.dataGridViewFiles.Rows[i];
                if (dr != null)
                    dr.Cells[1].Value = 1;
            }
        }
    }
}